*,
*::after,
*::before {
  margin: 0;
  padding: 0;
  box-sizing: inherit;
}

html {
    font-size: 62.5%;
}

body {
    position: relative;
    box-sizing: border-box;
    background: #ffffff;
}

.loader {
  position: absolute;
  top: 50%;
  left: 50%;
  transform: translate(-50%, -50%);
  font-size: 24px;
  background: rgba(255, 255, 255, 0.8);
  padding: 10px 20px;
  border-radius: 5px;
  display: flex;
  align-items: center;
  gap: 10px;
  white-space: nowrap; /* Prevents text from wrapping */
  min-width: fit-content; /* Ensures the container doesn’t shrink */
}

.spinner {
  width: 20px;
  height: 20px;
  border: 3px solid #ccc;
  border-top: 3px solid #007bff;
  border-radius: 50%;
  animation: spin 1s linear infinite;
}

.hidden {
  visibility: hidden;
}

::selection {
    background-color: #00b4e4;
    color: white;
}

.modal-wrapper {
    position: fixed;
    z-index: 100;
    top: 0;
    left: 0;
    width: 100vw;
    height: 100vh;
    display: flex;
    justify-content: center;
    align-items: center;
    align-content: center;
    background-color: rgba(0, 0, 0, 30%);
}

.modal {
    color: #444343;
    height: 280px;
    width: 250px;
    margin: 15px;
    background-color: #f7f7f7;
    border: 1px solid #ececec;
    border-radius: 3px;
}

.modal-heading {
    background-color: #f0f0f0;
    height: 20%;
    width: 100%;
    border-bottom: 1px solid #ececec;
    display: grid;
    grid-template-columns: 3fr 1fr;
}

.modal-title {
    display: flex;
    font-size: 15px;
    align-items: center;
    padding-left: 20px;
    font-family: poppins;
}

.modal-close-button {
    cursor: pointer;
    display: flex;
    justify-content: center;
    align-items: center;
}

.modal-content {
    padding: 20px;
}

#content_iframe {
    background: #ffffff;
}

#uiLanguageSelector {
  display: none;
}

#uiLanguageSelector .modal {
  height: 140px;
}

#uiLanguageSelector .modal-heading {
  height: 40%;
}

#uiLanguageSelector .modal-content #ui_language {
  font-size: 1.6rem;
  width: 100%;
}

#uiLanguageSelectorButton {
  margin: 0px 12px;
  float: right;
  cursor: pointer;
  height: 30px;
}

@keyframes spin {
  0% { transform: rotate(0deg); }
  100% { transform: rotate(360deg); }
}

@font-face {
  font-family: "poppins";
  src: url("../skin/fonts/Poppins.ttf?KIWIXCACHEID") format("truetype");
}

@font-face {
  font-family: "roboto";
  src: url("../skin/fonts/Roboto.ttf?KIWIXCACHEID") format("truetype");
}
