.kiwixNav {
    background-color: #f4f6f8;
    width: 100%;
    padding: 10px 20px;
    position: sticky;
    top: 0;
    z-index: 3;
    transition: all 0.5s ease;
}

.kiwixHomeBody__results {
    font-size: 1.6rem;
    color: #333333;
    font-family: poppins;
    margin: -10px 0 -5px;
    position: relative;
    top: -10px;
    left: 13px;
}

.kiwixNav__filters {
    display: grid;
    grid-gap: 20px;
    grid-template-columns: 231px 231px;
    margin: 0 0 10px;
}

.kiwixNav__kiwixFilter {
    -webkit-appearance: none;
    -moz-appearance: none;
    -ms-appearance: none;
    appearance: none;
    outline: 0;
    box-shadow: none;
    border: 0 !important;
    background: white;
    background-image: none;
    border-radius: 1px;
    width: 195px;
    height: 30px;
    flex: 1;
    color: black;
    padding: 0px 10px 0px 10px;
    cursor: pointer;
 }

 .kiwixNav__kiwixFilter:-ms-expand {
    display: none;
 }

.kiwixNav__select {
    position: relative;
    display: flex;
    width: 231px;
    height: 30px;
    line-height: 3;
    background: #909090;
    overflow: hidden;
    border-radius: 10px;
    border: 1px solid #b5b2b2;
}

.kiwixNav__select::after {
    content: '\25BC';
    color: #fff;
    background-color: #909090;
    height: 100%;
    position: absolute;
    top: 0;
    right: 0;
    padding: 0 1em;
    cursor: pointer;
    pointer-events: none;
    -webkit-transition: .25s all ease;
    -o-transition: .25s all ease;
    transition: .25s all ease;
}

.kiwixNav__kiwixFilter option {
    background-color: #fff;
    color: #000;
}

.kiwixSearch {
    height: 30px;
    width: 231px;
    border-radius: 10px;
    border: solid 1px #b5b2b2;
    padding: 10px;
    background-image: url('../skin/search-icon.svg?KIWIXCACHEID');
    background-repeat: no-repeat;
    background-position: right center;
    background-origin: content-box;
    background-size: 15px;
}

.kiwixButton {
    margin: 0 17px;
    height: 30px;
    width: 100px;
    border-radius: 10px;
    color: white;
    background-color: #909090;
    border: solid 1px #b5b2b2;
    cursor: pointer;
}

.kiwixButtonHover:hover {
    background-color: #858585;
}

.kiwixHomeBody {
    padding: 20px 0;
    min-height: calc(100vh - 170px);
    position: relative;
    display: flex;
    flex-direction: column;
    align-items: center;
    width: 100%;
}

.tagFilterLabel {
    width: max-content;
    padding: 7px;
    font-family: roboto;
    font-size: 12px;
    margin: 0 0 0 17px;
    text-align: center;
    background-color: transparent;
    color: #909090;
    display: none;
}

.tag__link {
    cursor: pointer;
    color: #909090;
}

.tag__link:hover {
    font-weight: bolder;
}

.book__list {
    position: relative;
    margin: 0 auto;
    padding-top: 20px;
}

.book {
    border-color: #acacac;
}

.book__link {
    text-decoration: none;
}

.book__wrapper {
    --tile-width: 250px;
    --tile-border-width:  1px;
    --tile-border-radius: 3px;
    color: #444343;
    height: 258px;
    width: var(--tile-width);
    margin: 15px;
    background-color: #f7f7f7;
    border: var(--tile-border-width) solid #ececec;
    border-radius: var(--tile-border-radius);
    display: flex;
    flex-direction: column;
    justify-content: space-between;
    transition: transform 0.25s;
    gap: 4px;
}

.book__wrapper:hover {
    transform: scale(1.07);
}

.book__link__wrapper {
    display: grid;
    grid-template-areas:
    "bookIcon bookHeader"
    "bookDesc bookDesc"
    ;
    grid-template-columns: 65px 1fr;
    grid-template-rows: 70px 120px;
}

.book__icon {
    background-image: url('data:image/png;base64, ');
    background-repeat: no-repeat;
    background-size: 100%;
    display: flex;
    align-items: center;
    align-content: center;
    justify-content: center;
    width:  48px;
    height: 48px;
    margin: 10px 0 0 10px;
    grid-area: bookIcon;
}

.book__header {
    display: grid;
    font-family: poppins;
    color: black;
    padding: 5px 10px 0 7px;
    width: 100%;
    height: 100%;
    align-items: center;
    align-content: center;
    grid-area: bookHeader;
}

.book__title {
    overflow: hidden;
    white-space: nowrap;
    text-overflow: ellipsis;
    font-size: 1.45rem;
}

.book__download {
    cursor: pointer;
    font-size: 1.2rem;
    background: #00b4e4;
    padding: 8px;
    border-radius: 0 0 var(--tile-border-radius) var(--tile-border-radius);
    display: flex;
    align-items: center;
    justify-content: center;
    position: relative;
    width: var(--tile-width);
    left: calc(0px - var(--tile-border-width));
    text-decoration: none;
}

.book__download > img {
    height: 16px;
    width: 16px;
}

.book__download > span {
    position: relative;
    font-family: roboto;
    color: white;
}

.book__download:hover {
    background: royalblue;
}

.book__description {
    margin: 10px 10px 5px;
    overflow: hidden;
    display: -webkit-box;
    -webkit-line-clamp: 7;
    -webkit-box-orient: vertical;
    font-family: roboto;
    font-weight: 300;
    font-size: 1.2rem;
    color: #4d4d4d;
    line-height: 1.25;
    grid-area: bookDesc;
}

.book__meta {
    display: flex;
    justify-content: space-between;
    padding: 0 10px 4px;
}

.book__languageTag {
    display: flex;
    align-items: center;
    justify-content: center;
    background-color: #ffffff;
    font-weight: bold;
    font-family: roboto;
    color: black;
    height: 25px;
    width: 25px;
    border-radius: 5px;
    font-size: 0.85rem;
}

.book__tags {
    display: flex;
    text-align: end;
    font-size: 1.1rem;
    justify-content: flex-end;
    font-family: roboto;
    overflow: hidden;
}

.book__tags--wrapper {
    overflow: hidden;
    display: -webkit-box;
    margin-top: auto;
    margin-bottom: auto;
    -webkit-line-clamp: 2;
    -webkit-box-orient: vertical;
}

.modal-content div {
    width: 100%;
    height: 40px;
}

.modal-content a {
    text-decoration: none;
}

.modal-content a>div {
    position: relative;
    top: -28px;
    left: 50px;
    text-decoration: underline;
    font-size: 13px;
    font-family: poppins;
    color: #444343;
}

.modal-content img {
    display: inline-block;
    margin: 0 10px;
    height: 30px;
}

.fadeOut {
    position: fixed;
    display: none;
    bottom: 0;
    left: 0;
    z-index: 1;
    background: linear-gradient(180deg, rgba(232, 232, 232, 0) 0%, #ffffff 100%);
    height: 80px;
    width: 100%;
}

.noResults {
    font-size: 1.6rem;
    text-align: center;
}

.noResults > a {
    color: #3498db;
}

.loader-spinner {
    position: absolute;
    top: -50%;
    left: 50%;
    border: 5px solid #f3f3f3;
    border-radius: 50%;
    border-top: 5px solid #3498db;
    width: 40px;
    height: 40px;
    margin: auto;
    -webkit-animation: spin 1s linear infinite; /* Safari */
    animation: spin 1s linear infinite;
    margin-top: 35px;
    margin-bottom: -35px;
    z-index: 2;
}
  /* Safari */
@-webkit-keyframes spin {
    0% { -webkit-transform: rotate(0deg); }
    100% { -webkit-transform: rotate(360deg); }
}

@keyframes spin {
    0% { transform: rotate(0deg); }
    100% { transform: rotate(360deg); }
}

.loader {
    display: none;
    position: relative;
    height: 70px;
    width: 100%;
}

.kiwixfooter {
    display: flex;
    width: 100%;
    justify-content: center;
    text-align: center;
    font-size: 1.6rem;
    height: 30px;
}

.kiwixfooter > a {
    width: auto;
}

.feedLogo {
    height: 30px;
    float: right;
}

@media screen and (max-width: 1100px) {

    .kiwixHomeBody {
        min-height: calc(100vh - 170px);
    }

    .kiwixNav__filters {
        grid-template-columns: repeat(4, 1fr);
    }

    .book__wrapper {
        margin: 10px;
    }
}

@media screen and (max-width: 590px) {

    .kiwixNav__SearchForm {
        display: flex;
        flex-direction: column;
    }

    .kiwixHomeBody {
        min-height: calc(100vh - 287px);
    }

    .kiwixSearch {
        margin-top: 11px;
    }

    .kiwixButton {
        margin: 15px 0;
        width: 229px;
    }

    .tagFilterLabel {
        margin: 15px 0 0 0;
    }

    .kiwixNav__filters {
        grid-template-columns: 1fr;
    }
}
